"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.useSyncDocument = exports.useSyncClient = void 0;
const react_1 = require("react");
const twilio_sync_1 = __importDefault(require("twilio-sync"));
const logger_1 = __importDefault(require("../utils/logger"));
function getClient(accessToken) {
    const logger = logger_1.default.child({
        module: 'useSyncClient/getClient',
    });
    if (!accessToken) {
        logger.debug('no sync access token yet');
        return null;
    }
    logger.debug({ msg: 'received access token', accessToken });
    return new twilio_sync_1.default(accessToken, {});
}
function useSyncClient(accessToken) {
    const syncClient = react_1.useMemo(() => getClient(accessToken), [accessToken]);
    return syncClient;
}
exports.useSyncClient = useSyncClient;
function useSyncDocument(documentId, accessToken) {
    const logger = logger_1.default.child({
        module: 'useSyncClient/useSyncDocument',
    });
    const [value, setValue] = react_1.useState(null);
    const [syncDoc, setSyncDoc] = react_1.useState(null);
    const [error, setError] = react_1.useState(null);
    const syncClient = useSyncClient(accessToken);
    const updateCallback = react_1.useCallback((data) => {
        logger.debug({
            msg: 'document update',
            data: data.value,
            isLocal: data.isLocal,
        });
        setValue(data.value);
    }, []);
    const removedCallback = react_1.useCallback((data) => {
        logger.debug({
            msg: 'document removed',
            isLocal: data.isLocal,
        });
        if (!data.isLocal) {
            logger.debug('resetting doc value to null');
            setValue(null);
        }
    }, []);
    react_1.useEffect(() => {
        return () => {
            if (syncDoc) {
                logger.debug({ msg: 'clean up document handlers' });
                syncDoc.off('updated', updateCallback);
                syncDoc.off('removed', removedCallback);
            }
        };
    }, []);
    react_1.useEffect(() => {
        if (syncClient && documentId && !syncDoc) {
            logger.debug({ msg: 'fetching document', documentId });
            syncClient
                .document(documentId)
                .then((doc) => {
                doc.on('updated', updateCallback);
                doc.on('removed', removedCallback);
                setSyncDoc(doc);
                logger.debug({ msg: 'received initial document', data: doc.value });
                setValue(doc.value);
            })
                .catch((err) => setError(err));
        }
    }, [syncClient, documentId]);
    return { value, syncClient, error };
}
exports.useSyncDocument = useSyncDocument;
