"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.installerMachine = exports.hasInstallStep = exports.hasSetupStep = exports.updateDirectory = void 0;
const xstate_1 = require("xstate");
const logger_1 = __importDefault(require("../utils/logger"));
const logger = logger_1.default.child({ stateMachine: 'installerMachine' });
function assignResult(name) {
    return xstate_1.assign({
        results: (context, event) => {
            return Object.assign(Object.assign({}, context.results), { [name]: event.data });
        },
    });
}
function assignResultAsSkipped(name, reason) {
    return xstate_1.assign({
        results: (context) => {
            return Object.assign(Object.assign({}, context.results), { [name]: { output: [`Skipped ${reason}`], skipped: true } });
        },
    });
}
const log = (context, event) => {
    logger.trace({ msg: 'action triggered', context, event });
};
const assignParsedResult = xstate_1.assign({
    results: (context, event) => {
        if (event.type === 'CONFIRM_CONFIGURING' && event.data) {
            return Object.assign(Object.assign({}, context.results), { parsedResult: event.data });
        }
    },
});
const assignDirectory = xstate_1.assign({
    directory: (context, event) => event.type === 'PROVIDE_DIRECTORY'
        ? event.data.directory
        : context.directory,
});
const handleError = xstate_1.assign({
    error: (context, event) => {
        logger.error({
            msg: 'Failure in installerMachine',
            event,
        });
        return [...context.error, event.data.message];
    },
});
exports.updateDirectory = xstate_1.assign({
    directory: (context, event) => event.data.fullPath,
    isEmptyDirectory: (context, event) => event.data.isEmpty,
});
function hasSetupStep(context, event) {
    var _a, _b, _c;
    return (((_a = event === null || event === void 0 ? void 0 : event.data) === null || _a === void 0 ? void 0 : _a.hasExampleEnvFile) || ((_c = (_b = context === null || context === void 0 ? void 0 : context.results) === null || _b === void 0 ? void 0 : _b.download) === null || _c === void 0 ? void 0 : _c.hasExampleEnvFile));
}
exports.hasSetupStep = hasSetupStep;
function hasInstallStep(context, event) {
    var _a, _b, _c, _d, _e, _f;
    const hasMakeFile = ((_a = event === null || event === void 0 ? void 0 : event.data) === null || _a === void 0 ? void 0 : _a.hasMakeFile) || ((_c = (_b = context === null || context === void 0 ? void 0 : context.results) === null || _b === void 0 ? void 0 : _b.download) === null || _c === void 0 ? void 0 : _c.hasMakeFile);
    const hasPackageJson = ((_d = event === null || event === void 0 ? void 0 : event.data) === null || _d === void 0 ? void 0 : _d.hasPackageJson) || ((_f = (_e = context === null || context === void 0 ? void 0 : context.results) === null || _e === void 0 ? void 0 : _e.download) === null || _f === void 0 ? void 0 : _f.hasPackageJson);
    return (context.language === 'TypeScript' ||
        context.language === 'JavaScript' ||
        hasMakeFile ||
        hasPackageJson);
}
exports.hasInstallStep = hasInstallStep;
// eslint-disable-next-line new-cap
exports.installerMachine = xstate_1.Machine({
    id: 'installer',
    initial: 'askingForDirectory',
    context: {
        directory: undefined,
        repoUrl: undefined,
        language: undefined,
        isEmptyDirectory: undefined,
        results: {},
        error: [],
    },
    states: {
        idle: {
            on: {
                START: 'askingForDirectory',
            },
        },
        askingForDirectory: {
            on: {
                PROVIDE_DIRECTORY: {
                    actions: [assignDirectory],
                    target: 'creatingDirectory',
                },
            },
        },
        creatingDirectory: {
            invoke: {
                id: 'createDirectory',
                src: 'createDirectory',
                onDone: {
                    target: 'confirmingDownload',
                    actions: [exports.updateDirectory, assignResult('createDirectory')],
                },
                onError: {
                    target: 'failure',
                    actions: [handleError],
                },
            },
        },
        confirmingDownload: {
            on: {
                CONFIRM: 'downloading',
                DENY: {
                    target: 'failure.cancelled',
                },
            },
        },
        downloading: {
            invoke: {
                id: 'download',
                src: 'download',
                onDone: [
                    {
                        target: 'confirmInstallingDependencies',
                        cond: hasInstallStep,
                        actions: [assignResult('download')],
                    },
                    {
                        target: 'confirmConfiguring',
                        cond: hasSetupStep,
                        actions: [
                            assignResult('download'),
                            assignResultAsSkipped('installDependencies', 'automatically. No installation for this project possible.'),
                        ],
                    },
                    {
                        target: 'success',
                        actions: [
                            log,
                            assignResult('download'),
                            assignResultAsSkipped('installDependencies', 'automatically. No installation for this project possible.'),
                            assignResultAsSkipped('configure', 'automatically. No automatic setup for this project possible.'),
                        ],
                    },
                ],
                onError: {
                    target: 'failure',
                    actions: [handleError],
                },
            },
        },
        confirmInstallingDependencies: {
            on: {
                CONFIRM: 'installingDependencies',
                DENY: {
                    target: 'confirmConfiguring',
                    actions: [assignResultAsSkipped('installDependencies', 'by user.')],
                },
            },
        },
        installingDependencies: {
            invoke: {
                id: 'installDependencies',
                src: 'installDependencies',
                onDone: [
                    {
                        cond: hasSetupStep,
                        target: 'confirmConfiguring',
                        actions: [assignResult('installDependencies')],
                    },
                    {
                        target: 'success',
                        actions: [
                            assignResult('installDependencies'),
                            assignResultAsSkipped('configure', 'automatically. No automatic setup for this project possible.'),
                        ],
                    },
                ],
                onError: {
                    target: 'failure',
                    actions: [handleError],
                },
            },
        },
        confirmConfiguring: {
            on: {
                CONFIRM_CONFIGURING: {
                    target: 'configuring',
                    actions: [assignParsedResult],
                },
                DENY: {
                    target: 'success',
                    actions: [assignResultAsSkipped('configure', 'by user.')],
                },
            },
        },
        configuring: {
            invoke: {
                id: 'configure',
                src: 'configure',
                onDone: {
                    target: 'success',
                    actions: [assignResult('configure')],
                },
                onError: {
                    target: 'failure',
                    actions: [handleError],
                },
            },
        },
        success: {
            type: 'final',
            data: {
                directory: (context) => context.directory,
                successful: true,
            },
            meta: { message: 'Your sample has been installed. Enjoy!' },
        },
        failure: {
            initial: 'error',
            states: {
                error: {
                    meta: { message: 'Something went wrong.' },
                    type: 'final',
                },
                cancelled: {
                    meta: { message: 'Feel free to start again.' },
                    on: {
                        RESTART: {
                            target: '#installer.askingForDirectory',
                            actions: xstate_1.assign({
                                directory: undefined,
                                isEmptyDirectory: undefined,
                                results: {},
                                error: [],
                            }),
                        },
                        FINISH: '#installer.finishedWithoutSuccess',
                    },
                },
            },
            meta: { message: 'Did not finish creating sample.' },
        },
        finishedWithoutSuccess: {
            type: 'final',
            meta: { message: 'Finished' },
            data: {
                directory: null,
                successful: false,
            },
        },
    },
});
