"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.setupProject = exports.getEnvSetupQuestions = exports.installNodeDependencies = exports.installDependenciesWithMake = exports.installDependencies = exports.download = exports.getAndCreateDirectory = void 0;
const output_1 = require("configure-env/dist/output");
const parser_1 = require("configure-env/dist/parser");
const degit_1 = __importDefault(require("degit"));
const execa_1 = __importDefault(require("execa"));
const fs_1 = __importDefault(require("fs"));
const mkdirp_1 = __importDefault(require("mkdirp"));
const path_1 = __importDefault(require("path"));
const pkg_install_1 = require("pkg-install");
const useTerminalInfo_1 = require("../hooks/useTerminalInfo");
const fsHelpers_1 = require("./fsHelpers");
const logger_1 = __importDefault(require("./logger"));
const fs = fs_1.default.promises;
function getAndCreateDirectory(relativePath) {
    return __awaiter(this, void 0, void 0, function* () {
        const fullPath = path_1.default.resolve(process.cwd(), relativePath);
        yield mkdirp_1.default(fullPath);
        const files = yield fs.readdir(fullPath);
        const output = files.length > 0
            ? `Found existing directory with files in it at ${fullPath}`
            : `Created empty directory at ${fullPath}`;
        return { fullPath, isEmpty: files.length === 0, output: [output] };
    });
}
exports.getAndCreateDirectory = getAndCreateDirectory;
function download(repoUrl, outputPath, options = {}) {
    return __awaiter(this, void 0, void 0, function* () {
        const logger = logger_1.default.child({ module: 'installer/download' });
        let { updateFunction, force } = options || {};
        force = typeof force === 'undefined' ? false : force;
        logger.info({ msg: 'cloning project', repoUrl, outputPath });
        const output = [];
        updateFunction =
            updateFunction ||
                ((msg) => {
                    logger.debug({ msg: 'cloning update', info: msg });
                    output.push(msg);
                });
        const emitter = degit_1.default(repoUrl, {
            cache: false,
            force: force,
            verbose: false,
        });
        emitter.on('info', updateFunction);
        yield emitter.clone(outputPath);
        emitter.off('info', updateFunction);
        const fullOutput = output.map((x) => x.message);
        const envExampleFilePath = path_1.default.join(outputPath, '.env.example');
        const hasExampleEnvFile = yield fsHelpers_1.exists(envExampleFilePath);
        const hasPackageJson = yield fsHelpers_1.exists(path_1.default.join(outputPath, 'package.json'));
        const terminalInfo = useTerminalInfo_1.getTerminalInfo();
        const hasMakeFile = terminalInfo.hasMake && (yield fsHelpers_1.exists(path_1.default.join(outputPath, 'Makefile')));
        const result = {
            output: [fullOutput[fullOutput.length - 1]],
            hasExampleEnvFile,
            hasMakeFile,
            hasPackageJson,
        };
        logger.info({ msg: 'finished cloning' });
        logger.debug({ msg: 'download result', result });
        return result;
    });
}
exports.download = download;
function installDependencies(projectPath, language, hasPackageJson, hasMakeFile) {
    return __awaiter(this, void 0, void 0, function* () {
        switch (language) {
            case 'JavaScript':
            case 'TypeScript':
                return installNodeDependencies(projectPath);
        }
        if (hasMakeFile) {
            return installDependenciesWithMake(projectPath);
        }
        if (hasPackageJson) {
            return installNodeDependencies(projectPath);
        }
        return Promise.reject(new Error('Invalid language'));
    });
}
exports.installDependencies = installDependencies;
function installDependenciesWithMake(projectPath) {
    return __awaiter(this, void 0, void 0, function* () {
        const logger = logger_1.default.child({ module: 'installer/install' });
        try {
            logger.debug({ msg: 'Running make install', projectPath });
            const { failed, stdout, stderr } = yield execa_1.default.command('make install', {
                shell: true,
                cwd: projectPath,
            });
            logger.debug({ failed, stdout, stderr, msg: 'make install result' });
            let summaryLine = 'Successfully installed dependencies';
            if (failed) {
                summaryLine =
                    'Failed to automatically install dependencies. Please try again manually.';
            }
            return { output: [summaryLine] };
        }
        catch (err) {
            logger.error({
                msg: 'failed to install dependencies with make',
                error: err,
            });
            logger.info('carrying on without dependency installation');
            return {
                output: [
                    'Failed to automatically install dependencies. Please try again manually.',
                ],
            };
        }
    });
}
exports.installDependenciesWithMake = installDependenciesWithMake;
function installNodeDependencies(projectPath) {
    return __awaiter(this, void 0, void 0, function* () {
        const logger = logger_1.default.child({ module: 'installer/install' });
        try {
            logger.debug({ msg: 'Running pkg-install.projectInstall', projectPath });
            const { failed, stdout, stderr } = yield pkg_install_1.projectInstall({
                cwd: projectPath,
            });
            logger.debug({ failed, stdout, stderr, msg: 'make install result' });
            let summaryLine = 'Successfully installed dependencies';
            if (failed) {
                summaryLine =
                    'Failed to automatically install dependencies. Please try again manually.';
            }
            return { output: [summaryLine] };
        }
        catch (err) {
            logger.error({
                msg: 'failed to install dependencies with pkgInstall.projectInstall',
                error: err,
            });
            logger.info('carrying on without dependency installation');
            return {
                output: [
                    'Failed to automatically install dependencies. Please try again manually.',
                ],
            };
        }
    });
}
exports.installNodeDependencies = installNodeDependencies;
const ALWAYS_SECRET_VALUES = [
    'TWILIO_AUTH_TOKEN',
    'AUTH_TOKEN',
    'TWILIO_API_SECRET',
    'API_SECRET',
];
function isVariableQuestion(value) {
    return typeof value !== 'undefined';
}
function createQuestionsFromVariableDeclarations(variableDeclarations, defaultValues) {
    return variableDeclarations
        .map((declaration) => {
        if (!declaration.configurable) {
            return undefined;
        }
        let type = 'text';
        if (declaration.format === 'secret' ||
            ALWAYS_SECRET_VALUES.includes(declaration.key)) {
            type = 'secret';
        }
        return {
            type,
            name: declaration.key,
            message: declaration.key,
            details: declaration.description,
            defaultValue: defaultValues[declaration.key] || declaration.default || undefined,
            earlyExitIfValue: undefined,
        };
    })
        .filter(isVariableQuestion);
}
function getEnvSetupQuestions(projectPath, defaultValues) {
    return __awaiter(this, void 0, void 0, function* () {
        const logger = logger_1.default.child({
            module: 'installer/getEnvSetupQuestions',
        });
        const envExampleFilePath = path_1.default.join(projectPath, '.env.example');
        if (yield fsHelpers_1.exists(envExampleFilePath)) {
            try {
                const result = yield parser_1.parseFile(envExampleFilePath);
                const questions = createQuestionsFromVariableDeclarations(result.variables, defaultValues);
                return Object.assign(Object.assign({}, result), { questions });
            }
            catch (err) {
                logger.error({ msg: 'failed to parse env file', error: err });
                logger.info('carrying on without setup questions');
                return null;
            }
        }
        return null;
    });
}
exports.getEnvSetupQuestions = getEnvSetupQuestions;
function setupProject(projectPath, parsedExample, answers) {
    return __awaiter(this, void 0, void 0, function* () {
        const logger = logger_1.default.child({ module: 'installer/setupProject' });
        const outputEnvFile = path_1.default.join(projectPath, '.env');
        if (yield fsHelpers_1.exists(outputEnvFile)) {
            return {
                output: [
                    `Existing .env file found at ${outputEnvFile}. Please configure manually.`,
                ],
            };
        }
        const output = output_1.createOutput(parsedExample, answers);
        try {
            yield fs.writeFile(outputEnvFile, output, 'utf8');
            return {
                output: [`Created config file at ${outputEnvFile}`],
            };
        }
        catch (err) {
            logger.error({ msg: 'failed to write new .env file', error: err });
            logger.info('carrying on without creating file');
            return {
                output: [`Failed to create .env file. Manual configuration necessary.`],
            };
        }
    });
}
exports.setupProject = setupProject;
